#!/usr/bin/env python

"""
WSGI adapter.
"""

import WebStack.WSGI
from WebStack.Generic import EndOfResponse

class WSGIAdapter:

    "A WSGI adapter class."

    def __init__(self, resource, authenticator=None):

        """
        Initialise the adapter with the given WebStack 'resource' and the
        optional 'authenticator'.
        """

        self.resource = resource
        self.authenticator = authenticator

    def __call__(self, environ, start_response):

        """
        Dispatch to the root application-specific 'resource'. Return a list of
        strings comprising the response body text.
        """

        trans = WebStack.WSGI.Transaction(environ)

        try:
            if self.authenticator is None or self.authenticator.authenticate(trans):
                try:
                    self.resource.respond(trans)
                except EndOfResponse:
                    pass
            else:
                trans.set_response_code(401) # Unauthorized
                trans.set_header_value("WWW-Authenticate", '%s realm="%s"' % (
                    self.authenticator.get_auth_type(), self.authenticator.get_realm()))
        finally:
            trans.commit()

        # NOTE: Provide sensible messages.

        start_response(
            "%s WebStack status" % trans.get_response_code(),
            trans.get_wsgi_headers()
            )
        return [trans.get_wsgi_content()]

# vim: tabstop=4 expandtab shiftwidth=4
